# Supports Organizer.rb
=begin
Copyright 2009, Matt
All Rights Reserved

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License:   Free

Author:     	Matthieu NOBLET
Organization:   Matt
Name:      		Selection manager
Version:   		1.1
SU Version:   	6 and 7
Date:      		17/03/2009
Description:   	Manage selection
Usage:      	
History:
	1.000   13/03/2009
			First version
	1.100	17/03/2009
			Add circles and arcs
=end

require 'sketchup.rb'

unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
	organizer.add_separator
	organizer.add_item("Selection manager"){ Matt_Tools.selection_manager }
	organizer.add_item("Selected edges by defined angle"){ Matt_Selection.sca 0 }
	organizer.add_item("Selected edges by user-defined angle"){ Matt_Selection.sca 1 }
	
	UI.add_context_menu_handler do |menu|
		menu.add_item("Connected edges by angle"){ Matt_Selection.sca 0 } if Matt_Selection.SCA_valid?
		menu.add_item("Selection manager"){Matt_Tools.selection_manager}
	end
end

module Matt_Tools
	def self.selection_manager
		model = Sketchup.active_model
		sel = model.selection
		
		if sel.empty?
			return nil
		else
			l_sel = sel.to_a
			l_ents=[];l_actions=[]
			l_sel.each do |e|
				if e.respond_to?("typename")
					if e.respond_to? "curve"
						if e.curve.start_angle==0.0 && e.curve.end_angle==Math::PI*2
							l_ents.push "Circle"
						else
							l_ents.push "Arc"
						end
						l_sel.delete e.curve.edges
					else
						l_ents.push(e.typename)
					end
				end
			end
			l_ents=l_ents.push("All").flatten.sort.uniq if l_ents.length>1
			ci = true if l_ents.index("ComponentInstance")
			l_actions.push("Get center point","Remove center point") if l_ents.index("Circle") or l_ents.index("Arc")
			l_actions.push "Add same instances" if ci
			l_actions.push "Add same ents","Erase","Hide","Hide on other scenes","Hide other ents","Inverse","Move on new layer","Remove","Remove other ents"
			if sel.count==1 and l_ents.length==1 and l_ents[0]=="Edge"
				l_actions.push "Add connected edges","Add edges by defined angle","Add edges by user-defined angle"
				["Erase","Hide","Inverse","Remove","Remove other ents"].each{|a|l_actions.delete a}
			end
			l_actions.flatten.sort!
			prompts = ["Entities: ","Action: "]
			if $Matt_Tools_selection_manager_ent and l_ents.index($Matt_Tools_selection_manager_ent)
				ent=$Matt_Tools_selection_manager_ent
				if $Matt_Tools_selection_manager_action!="Add same instances" or ($Matt_Tools_selection_manager_action=="Add same instances" and ci)
					action=$Matt_Tools_selection_manager_action
				end
			else
				if l_ents[1];ent=l_ents[1];else;ent=l_ents[0];end
				action="Add same entities" 
			end
			values = [ent,action]
			infos = [l_ents.sort.join('|').to_s,l_actions.sort.join('|')]
			results = inputbox prompts, values, infos, "Selection manager"
			return if not results
			ent=$Matt_Tools_selection_manager_ent=results[0]
			action=$Matt_Tools_selection_manager_action=results[1]
			
			if Sketchup.version[0,1].to_i >= 7
				model.start_operation("Selection manager",true)
			else
				model.start_operation "Selection manager"
			end
			if ["Circle","Arc"].index ent
				curve_l=[]
				sel.select{|e|curve_l.push e if e.respond_to? "curve"}
				curve_l.each{|e|e.curve.edges[1..-1].each{|elt|curve_l.delete elt}}
			end
			case action
				when "Get center point"
					if ent=="Circle"
						curve_l.each{|e|p=model.entities.add_cpoint(e.curve.center) if e.curve.start_angle==0.0 && e.curve.end_angle==Math::PI*2}
					elsif ent=="Arc"
						curve_l.each{|e|p=model.entities.add_cpoint(e.curve.center) if (not e.curve.start_angle==0.0 && e.curve.end_angle==Math::PI*2)}
					else
						return nil
					end
				when "Remove center point"
					if ent=="Circle"
						sel.select{|e|e.class==Sketchup::ConstructionPoint}.each do |e|
							curve_l.each do |elt|
								e.erase! if e.valid? && e.position==elt.curve.center && elt.curve.start_angle==0.0 && elt.curve.end_angle==Math::PI*2
							end
						end
					elsif ent=="Arc"
						sel.select{|e|e.class==Sketchup::ConstructionPoint}.each do |e|
							curve_l.each do |elt|
								 e.erase! if e.valid? && e.position==elt.curve.center and not(elt.curve.end_angle==Math::PI*2)
							end
						end
					else
						return nil
					end
				when "Add same ents"
					if ent=="All"
						l_ents.each do |elt|
							sel.add(model.active_entities.select{|e|e.typename==elt})
						end
					else
					sel.add(model.active_entities.select{|e|e.typename==ent})
				end				
				when "Erase"
					if ent=="All";sel.each{|e|e.erase!};else;sel.each{|e|e.erase! if e.typename==ent};end			
				when "Add same instances"
					l = []
					sel.each{|e|l.push e.definition if e.class==Sketchup::ComponentInstance}
					l.uniq.each do |elt|
						sel.add(model.active_entities.select{|e|e.typename==ent && e.definition==elt})
					end
				when "Hide"
					if ent=="All";sel.each{|e|e.hidden=true};else;sel.each{|e|e.hidden=true if e.typename==ent};end
				when "Inverse"
					model.active_entities.each{|e|sel.toggle(e)}
				when "Hide other ents"
					model.active_entities.each{|e|e.hidden=true}
					if ent=="All"
						sel.each{|e|e.hidden=false}
					else
						sel.each{|e|if e.typename==ent;e.hidden=false;else;sel.remove e;end }
					end
				when "Remove other ents";if ent=="All";return nil;else;sel.each{|e|sel.remove e if e.typename!=ent};end
				when "Move on new layer"
					l = model.layers
					result = inputbox(["name: "], [l.unique_name], "new layer")
					return unless result
					name = result[0].strip
					if model.layers[name]
						UI.messagebox("Existing layer \"#{name}\"!")
						return nil
					end
					l = l.add name
					if ent=="All";sel.each{|e|e.layer=l};else;sel.each{|e|e.layer=l if e.typename==ent};end
				when "Remove"
					if ent=="All";sel.clear;else;sel.each{|e|sel.remove e if e.typename==ent};end
				when "All connected edges"
					sel.add sel[0].all_connected.select{|e|e.class==Sketchup::Edge}
				when "Add edges by defined angle";Matt_Selection.sca 0
				when "Add edges by user-defined angle";Matt_Selection.sca 1
			end
			model.active_view.invalidate
			model.commit_operation
		end
	end
end
def sm;Matt_Tools.selection_manager;end

module Matt_Selection
def self.sca(int)
	def self.loop(e,all_c,l)
		le = []
		le.push e.start.position,e.end.position
		if lst = all_c.select{|elt|le.index(elt.start.position) or le.index(elt.end.position) and elt!=e}
			for elt in lst
				l_temp = le + [elt.start.position,elt.end.position]
				inters = l_temp.get_double
				l_temp.delete inters
				an = l_temp[0].vector_to(inters).angle_between(inters.vector_to(l_temp[1]))
				if an <= $select_connected_edges_angle.degrees
					l.push elt
					all_c.delete elt
					self.loop(elt,all_c,l)
				end
				
			end
		end
	end
	model = Sketchup.active_model
	sel = model.selection
	if sel.empty? or sel.length!=1
		Sketchup.set_status_text "You must select just one edge...", SB_PROMPT
		return nil
	elsif sel[0].class==Sketchup::Edge
		if int==0
			$select_connected_edges_angle=25
		else
			prompt = ["Max angle (degrees) : "]
			if $select_connected_edges_angle
				value = [$select_connected_edges_angle]
			else
				value = [25]
			end
			results = inputbox prompt, value, "Maximum connected angle"
			return if not results
			$select_connected_edges_angle = results[0].to_i
		end
		puts("Current angle: " + $select_connected_edges_angle.to_s + ".")
		ent = sel[0]
		all_c = ent.all_connected.select{|e|e.typename == "Edge"}.to_a
		all_c.delete ent
		l = []
		Sketchup.set_status_text "Program is running...", SB_PROMPT
		self.loop(ent,all_c,l)
		Sketchup.set_status_text "Done!", SB_PROMPT
		if l.length>0
			puts(l.length.to_s + " entities added to the current selection.")
			sel.add l.push(ent)
		end
	end
end
def self.SCA_valid?
	ss = Sketchup.active_model.selection
	return true if ss.length==1 and ss[0].class==Sketchup::Edge
	return false
end
end

# Return first doublon in a list
class Array 
	def get_double
		l = self
		while l.length>0
			elt = l.first
			return l.first if(l[1..-1].index elt)
			l = l[1..-1]
		end
	end
end
